@@ -189,7 +189,9 @@ void *mm_core_create(size_t usersize, const char *file)
 {
     mem_core *mc;
     void *area = ((void *)-1);
+#if defined(MM_SHMT_MMPOSX) || defined(MM_SHMT_MMZERO) || defined(MM_SHMT_MMFILE) || defined(MM_SHMT_IPCSHM)
     int fdmem = -1;
+#endif
     int fdsem = -1;
 #if defined(MM_SEMT_IPCSEM)
     int fdsem_rd = -1;
@@ -262,22 +264,20 @@ void *mm_core_create(size_t usersize, const char *file)
                              MAP_SHARED, fdmem, mm_core_mapoffset)) == (void *)MAP_FAILED)
         FAIL(MM_ERR_CORE|MM_ERR_SYSTEM, "failed to memory map tempfile");
     shm_unlink(fnmem);
+    close(fdmem); fdmem = -1;
     mm_core_mapoffset += size;
 #endif /* MM_SHMT_MMPOSX */
 
 #if defined(MM_SHMT_MMZERO)
     if ((fdmem = open("/dev/zero", O_RDWR, MM_CORE_FILEMODE)) == -1)
         FAIL(MM_ERR_CORE|MM_ERR_SYSTEM, "failed to open /dev/zero");
-#if defined(F_SETFD) && defined(FD_CLOEXEC)
-    if (fcntl(fdmem, F_SETFD, FD_CLOEXEC) == -1)
-        FAIL(MM_ERR_CORE|MM_ERR_SYSTEM, "failed to set close-on-exec flag");
-#endif
     if (lseek(fdmem, mm_core_mapoffset+size, SEEK_SET) == -1)
         FAIL(MM_ERR_CORE|MM_ERR_SYSTEM, "failed to seek in /dev/zero");
     write(fdmem, &zero, sizeof(zero));
     if ((area = (void *)mmap(NULL, size, PROT_READ|PROT_WRITE,
                              MAP_SHARED, fdmem, mm_core_mapoffset)) == (void *)MAP_FAILED)
         FAIL(MM_ERR_CORE|MM_ERR_SYSTEM, "failed to memory map /dev/zero");
+    close(fdmem); fdmem = -1;
     mm_core_mapoffset += size;
 #endif /* MM_SHMT_MMZERO */
 
@@ -285,16 +285,13 @@ void *mm_core_create(size_t usersize, const char *file)
     unlink(fnmem);
     if ((fdmem = open(fnmem, O_RDWR|O_CREAT|O_EXCL, MM_CORE_FILEMODE)) == -1)
         FAIL(MM_ERR_CORE|MM_ERR_SYSTEM, "failed to open memory file");
-#if defined(F_SETFD) && defined(FD_CLOEXEC)
-    if (fcntl(fdmem, F_SETFD, FD_CLOEXEC) == -1)
-        FAIL(MM_ERR_CORE|MM_ERR_SYSTEM, "failed to set close-on-exec flag");
-#endif
     if (ftruncate(fdmem, mm_core_mapoffset+size) == -1)
         FAIL(MM_ERR_CORE|MM_ERR_SYSTEM, "failed to truncate memory file");
     write(fdmem, &zero, sizeof(zero));
     if ((area = (void *)mmap(NULL, size, PROT_READ|PROT_WRITE,
                              MAP_SHARED, fdmem, mm_core_mapoffset)) == (void *)MAP_FAILED)
         FAIL(MM_ERR_CORE|MM_ERR_SYSTEM, "failed to memory map memory file");
+    close(fdmem); fdmem = -1;
     mm_core_mapoffset += size;
 #endif /* MM_SHMT_MMFILE */
 
@@ -357,7 +354,9 @@ void *mm_core_create(size_t usersize, const char *file)
     mc->mc_size     = size;
     mc->mc_usize    = usersize;
     mc->mc_pid      = getpid();
+#if defined(MM_SHMT_IPCSHM)
     mc->mc_fdmem    = fdmem;
+#endif
 #if defined(MM_SEMT_FLOCK)
     mc->mc_fdsem[0].pid = getpid();
     mc->mc_fdsem[0].fd  = fdsem;
@@ -401,7 +400,7 @@ void *mm_core_create(size_t usersize, const char *file)
     if (area != ((void *)-1))
         shmdt(area);
 #endif
-#if defined(MM_SHMT_MMPOSX) || defined(MM_SHMT_MMFILE)
+#if defined(MM_SHMT_MMPOSX) || defined(MM_SHMT_MMZERO) || defined(MM_SHMT_MMFILE)
     if (fdmem != -1)
         close(fdmem);
 #endif
@@ -489,7 +488,9 @@ int mm_core_permission(void *core, mode_t mode, uid_t owner, gid_t group)
 void mm_core_delete(void *core)
 {
     mem_core *mc;
+#if defined(MM_SHMT_IPCSHM)
     int fdmem;
+#endif
     int fdsem;
 #if defined(MM_SEMT_IPCSEM)
     int fdsem_rd;
@@ -506,7 +507,9 @@ void mm_core_delete(void *core)
         return;
     mc = (mem_core *)((char *)core-SIZEOF_mem_core);
     size  = mc->mc_size;
+#if defined(MM_SHMT_IPCSHM)
     fdmem = mc->mc_fdmem;
+#endif
 #if !defined(MM_SEMT_FLOCK)
     fdsem = mc->mc_fdsem;
 #endif
@@ -530,9 +533,6 @@ void mm_core_delete(void *core)
     shmdt((void *)mc);
     shmctl(fdmem, IPC_RMID, NULL);
 #endif
-#if defined(MM_SHMT_MMPOSX) || defined(MM_SHMT_MMZERO) || defined(MM_SHMT_MMFILE)
-    close(fdmem);
-#endif
 #if defined(MM_SHMT_MMFILE)
     unlink(fnmem);
 #endif
